package de.duehl.math.graph.ged.animation;

import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.graph.edge.Edge;
import de.duehl.math.graph.ged.graph.vertex.Vertex;

public class WayFindAlgorithmGraph extends AlgorithmGraph {

    private static final GedColor START_VERTEX_COLOR = GedColor.AZURE;
    private static final int START_VERTEX_RADIUS = 10;

    private static final GedColor TARGET_VERTEX_COLOR = GedColor.BURLYWOOD;
    private static final int TARGET_VERTEX_RADIUS = 10;

    private static final int WAY_EDGE_WIDTH = 3;
    private static final GedColor WAY_EDGE_COLOR = GedColor.RED;

    public void setStartVertex(int vertexIndex) {
        if (underConstruction) {
            Vertex vertex = graph.getVertex(vertexIndex);
            vertex.setColor(START_VERTEX_COLOR);
            vertex.setRadius(START_VERTEX_RADIUS);
        }
        else {
            throw new RuntimeException("Nicht mehr in der Konstrutkionsphase!");
        }
    }

    public void setTargetVertex(int vertexIndex) {
        if (underConstruction) {
            Vertex vertex = graph.getVertex(vertexIndex);
            vertex.setColor(TARGET_VERTEX_COLOR);
            vertex.setRadius(TARGET_VERTEX_RADIUS);
        }
        else {
            throw new RuntimeException("Nicht mehr in der Konstrutkionsphase!");
        }
    }

    public void markEdgeAsPartOfTheWay(int startVertexIndex, int targetVertexIndex) {
        if (underConstruction) {
            throw new RuntimeException("Noch in der Konstrutkionsphase!");
        }
        else {
            Edge edge = graph.getEdge(startVertexIndex, targetVertexIndex);
            edge.setWidth(WAY_EDGE_WIDTH);
            edge.setColor(WAY_EDGE_COLOR);
            storeStep();
        }
    }

    public void markEdgeAsNotPartOfTheWay(int startVertexIndex, int targetVertexIndex) {
        if (underConstruction) {
            throw new RuntimeException("Noch in der Konstrutkionsphase!");
        }
        else {
            Edge edge = graph.getEdge(startVertexIndex, targetVertexIndex);
            edge.setWidthToDefault();
            edge.setColorToDefault();
            storeStep();
        }
    }

}
