package de.duehl.math.graph.ged.ui.creation.menu;

import java.awt.event.InputEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JMenuBar;

import de.duehl.math.graph.ged.logic.Logic;
import de.duehl.math.graph.ged.ui.GedGui;
import de.duehl.math.graph.ged.ui.Gui;
import de.duehl.math.graph.ged.ui.GuiElements;
import de.duehl.swing.ui.menu.SingleMenuCreation;

/**
 * Diese Klasse erzeugt die Menü-Elemente der grafische Oberfläche des Hauptprogramms.
 *
 * @version 1.01     2019-05-22
 * @author Christian Dühl
 */

public class MenuCreation {

    /** Zugriff auf die Programmlogik */
    private final Logic logic;

    /** Verbindung zur Gui. */
    private final Gui gui;

    /** Enthält die grafischen Elemente. */
    private final GuiElements elements;

    /** Das Menü. */
    private final JMenuBar menuBar;

    /** Tastaturmaske für die Control-Taste. */
    private final int ctrlMask;

    /** Tastaturmaske für die Shift-Taste und die Control-Taste gleichzeitig. */
    private final int shiftCtrlMask;

    /** Tastaturmaske für die Alt-Taste. */
    private final int altMask;

    /**
     * Konstruktor
     *
     * @param logic
     *            Der Frame in dem die grafischen Elemente erzeugt werden.
     * @param gui
     *            Zugriff auf die Programmlogik
     */
    public MenuCreation(Logic logic, GedGui gui, GuiElements elements) {
        this.logic =  logic;
        this.gui =  gui;
        this.elements =  elements;

        menuBar = new JMenuBar();
        elements.setMenuBar(menuBar);

        ctrlMask = InputEvent.CTRL_DOWN_MASK;
        shiftCtrlMask = ctrlMask | InputEvent.SHIFT_DOWN_MASK;
        altMask =  InputEvent.ALT_DOWN_MASK;

        createTheMenu();
    }

    /** Erzeugt das Menü. */
    private void createTheMenu() {
        List<SingleMenuCreation> menues = new ArrayList<>();
        menues.add(new FileMenuCreation(logic, gui, ctrlMask, shiftCtrlMask, altMask));
        menues.add(new EditMenuCreation(logic, elements, ctrlMask));
        menues.add(new ViewMenuCreation(logic));
        menues.add(new ExtraMenuCreation(logic));
        menues.add(new HelpMenuCreation(logic, gui));

        menuBar.removeAll();

        for (SingleMenuCreation menu : menues) {
            menuBar.add(menu.createTheMenu());
        }
    }

    /**
     * Trägt das Menü in einem JFrame ein.
     *
     * @param frame
     *            JFrame, bei dem das Menü eingetragen werden soll.
     */
    public void addTo(JFrame frame) {
        frame.setJMenuBar(menuBar);
        frame.validate();
    }

}
