package de.duehl.math.graph.ged.graph.edge;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.Test;

import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.pixel.PixelPoint;

public class EdgeTest {

    @Test
    public void createEmptyEdge() {
        Edge edge = new Edge();
        assertNotNull(edge);
    }

    @Test
    public void createEmptyEdgeToString() {
        Edge edge = new Edge();

        String excpected = "Edge [\n"
                + "    startVertexIndex=-1,\n"
                + "    targetVertexIndex=-1,\n"
                + "    color=Color [red=-1, blue=-1, green=-1],\n"
                + "    style=NORMAL,\n"
                + "    width=-17,\n"
                + "    label=,\n"
                + "    labelDistance=PixelPoint [x=-20, y=-20],\n"
                + "    labelColor=Color [red=-1, blue=-1, green=-1],\n"
                + "    weight=-Infinity,\n"
                + "    weightDistance=PixelPoint [x=20, y=20],\n"
                + "    weightColor=Color [red=-1, blue=-1, green=-1],\n"
                + "    capacity=-Infinity,\n"
                + "    capacityDistance=PixelPoint [x=-20, y=20],\n"
                + "    capacityColor=Color [red=-1, blue=-1, green=-1],\n"
                + "    flow=-Infinity,\n"
                + "    flowDistance=PixelPoint [x=20, y=-20],\n"
                + "    flowColor=Color [red=-1, blue=-1, green=-1]\n"
                + "]";
        String actual = edge.toString();
        assertEquals(excpected, actual);
    }

    @Test
    public void addLabelAndCapacityAndDoToString() {
        Edge edge = new Edge();
        edge.setLabel("foo");
        edge.setCapacity(22D);

        String excpected = "Edge [\n"
                + "    startVertexIndex=-1,\n"
                + "    targetVertexIndex=-1,\n"
                + "    color=Color [red=-1, blue=-1, green=-1],\n"
                + "    style=NORMAL,\n"
                + "    width=-17,\n"
                + "    label=foo,\n"
                + "    labelDistance=PixelPoint [x=-20, y=-20],\n"
                + "    labelColor=Color [red=-1, blue=-1, green=-1],\n"
                + "    weight=-Infinity,\n"
                + "    weightDistance=PixelPoint [x=20, y=20],\n"
                + "    weightColor=Color [red=-1, blue=-1, green=-1],\n"
                + "    capacity=22.0,\n"
                + "    capacityDistance=PixelPoint [x=-20, y=20],\n"
                + "    capacityColor=Color [red=-1, blue=-1, green=-1],\n"
                + "    flow=-Infinity,\n"
                + "    flowDistance=PixelPoint [x=20, y=-20],\n"
                + "    flowColor=Color [red=-1, blue=-1, green=-1]\n"
                + "]";
        String actual = edge.toString();
        assertEquals(excpected, actual);
    }

    @Test
    public void addLabelCapacityAndFlowAndDoToString() {
        Edge edge = new Edge();
        edge.setLabel("foo");
        edge.setCapacity(22D);
        edge.setCapacityColor(GedColor.AQUAMARINE);
        edge.setCapacityDistance(new PixelPoint(20, 55));
        edge.setFlow(17.7765D);
        edge.setFlowColor(GedColor.DARK_ORANGE);
        edge.setFlowDistance(new PixelPoint(-20, 35));

        String excpected = "Edge [\n"
                + "    startVertexIndex=-1,\n"
                + "    targetVertexIndex=-1,\n"
                + "    color=Color [red=-1, blue=-1, green=-1],\n"
                + "    style=NORMAL,\n"
                + "    width=-17,\n"
                + "    label=foo,\n"
                + "    labelDistance=PixelPoint [x=-20, y=-20],\n"
                + "    labelColor=Color [red=-1, blue=-1, green=-1],\n"
                + "    weight=-Infinity,\n"
                + "    weightDistance=PixelPoint [x=20, y=20],\n"
                + "    weightColor=Color [red=-1, blue=-1, green=-1],\n"
                + "    capacity=22.0,\n"
                + "    capacityDistance=PixelPoint [x=20, y=55],\n"
                + "    capacityColor=Color [red=127, blue=212, green=255],\n"
                + "    flow=17.7765,\n"
                + "    flowDistance=PixelPoint [x=-20, y=35],\n"
                + "    flowColor=Color [red=255, blue=0, green=140]\n"
                + "]";
        String actual = edge.toString();
        assertEquals(excpected, actual);
    }

}
