package de.duehl.math.graph.ged.graph.vertex;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.Test;

public class VertexTest {

    @Test
    public void createEmptyVertex() {
        Vertex vertex = new Vertex();
        assertNotNull(vertex);
    }

    @Test
    public void createVertexWithLabel() {
        Vertex vertex = new Vertex();
        vertex.setLabel("Eine Ecke");
        assertNotNull(vertex);
    }

    @Test
    public void createVertexWithFormOtherVertex() {
        Vertex vertex = new Vertex();
        vertex.setLabel("Eine Ecke");

        Vertex copiedVertex = new Vertex(vertex);
        assertNotNull(copiedVertex);
    }

    @Test
    public void createVertexWithLabelTextToString() {
        Vertex vertex = new Vertex();
        vertex.setLabel("Ecke fünf");
        String expected = "Vertex [\n"
                + "    index=-1,\n"
                + "    coordinates=PixelPoint [x=0, y=0],\n"
                + "    radius=-16,\n"
                + "    color=Color [red=-1, blue=-1, green=-1],\n"
                + "    label=Ecke fünf,\n"
                + "    labelDistance=PixelPoint [x=0, y=0],\n"
                + "    labelColor=Color [red=-1, blue=-1, green=-1],\n"
                + "    weight=-Infinity,\n"
                + "    weightDistance=PixelPoint [x=0, y=0],\n"
                + "    weightColor=Color [red=-1, blue=-1, green=-1]\n"
                + "]";
        String actual = vertex.toString();
        assertEquals(expected, actual);
    }

    @Test
    public void createVertexWithFormOtherVertexAndTestToString() {
        Vertex vertex = new Vertex();
        vertex.setLabel("Eine Ecke");

        Vertex copiedVertex = new Vertex(vertex);
        assertNotNull(copiedVertex);

        vertex.setLabel("Ecke fünf");

        String expectedOriginal = "Vertex [\n"
                + "    index=-1,\n"
                + "    coordinates=PixelPoint [x=0, y=0],\n"
                + "    radius=-16,\n"
                + "    color=Color [red=-1, blue=-1, green=-1],\n"
                + "    label=Ecke fünf,\n"
                + "    labelDistance=PixelPoint [x=0, y=0],\n"
                + "    labelColor=Color [red=-1, blue=-1, green=-1],\n"
                + "    weight=-Infinity,\n"
                + "    weightDistance=PixelPoint [x=0, y=0],\n"
                + "    weightColor=Color [red=-1, blue=-1, green=-1]\n"
                + "]";
        String expectedCopied = "Vertex [\n"
                + "    index=-1,\n"
                + "    coordinates=PixelPoint [x=0, y=0],\n"
                + "    radius=-16,\n"
                + "    color=Color [red=-1, blue=-1, green=-1],\n"
                + "    label=Eine Ecke,\n"
                + "    labelDistance=PixelPoint [x=0, y=0],\n"
                + "    labelColor=Color [red=-1, blue=-1, green=-1],\n"
                + "    weight=-Infinity,\n"
                + "    weightDistance=PixelPoint [x=0, y=0],\n"
                + "    weightColor=Color [red=-1, blue=-1, green=-1]\n"
                + "]";
        String actualOriginal = vertex.toString();
        String actualCopied = copiedVertex.toString();
        assertEquals(expectedOriginal, actualOriginal);
        assertEquals(expectedCopied, actualCopied);
    }

    @Test
    public void equalsItself() {
        Vertex vertex = new Vertex();
        vertex.setLabel("Eine Ecke");

        assertEquals(vertex, vertex);
    }

    @Test
    public void equalsWithSameValues() {
        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Eine Ecke");

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Eine Ecke");

        assertEquals(vertex1, vertex2);
    }

    @Test
    public void notEqualsWithDifferentValues() {
        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Eine Ecke");

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Eine andere Ecke");

        assertFalse(vertex1.equals(vertex2));
    }

    @Test
    public void notEqualsWithSameValuesButDifferentIndices() {
        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Eine Ecke");
        vertex1.setIndex(17);

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Eine Ecke");
        vertex2.setIndex(18);

        assertFalse(vertex1.equals(vertex2));
    }

}
