package de.duehl.math.graph.ged.ui.elements;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;

import de.duehl.math.graph.ged.ui.GuiElements;
import de.duehl.math.graph.ged.ui.graphpanel.EditModus;
import de.duehl.math.graph.ged.ui.graphpanel.GraphPanel;

/**
 * Diese Klasse stellt einen MouseListener für die Schalter für den gewählten Bearbeitungsmodus
 * dar.
 *
 * @version 1.01     2019-05-22
 * @author Christian Dühl
 */

public class ModusButtonMouseListener implements MouseListener {

    public static final Border LOWERED_BORDER = BorderFactory
            .createBevelBorder(BevelBorder.LOWERED);
    public static final Border RAISED_BORDER = BorderFactory.createBevelBorder(BevelBorder.RAISED);

    private final EditModus modus;
    private final GuiElements elements;

    public ModusButtonMouseListener(EditModus modus, GuiElements elements) {
        this.modus = modus;
        this.elements = elements;
    }

    @Override
    public void mouseClicked(MouseEvent event) {
    }

    @Override
    public void mouseEntered(MouseEvent event) {
        elements.getStatusLabel().setText(modus.getDescription());
    }

    @Override
    public void mouseExited(MouseEvent event) {
        elements.getStatusLabel().setText("");
    }

    @Override
    public void mousePressed(MouseEvent event) {
        GraphPanel graphPanel = elements.getGraphPanel();
        graphPanel.setModus(modus);
        setDescription(modus);
        for (EditModus eachModus : EditModus.values()) {
            eachModus.setButtonBorder(RAISED_BORDER);
        }
        modus.setButtonBorder(LOWERED_BORDER);
    }

    /** Setzt die Erklärung des Bearbeitungsmodus. */
    private void setDescription(EditModus modus) {
        JLabel modusDescription = elements.getModusDescriptionLabel();
        modusDescription.setText(modus.getDescription());
    }

    @Override
    public void mouseReleased(MouseEvent event) {
    }

}
