package de.duehl.math.graph.ged.ui.graphpanel;

import java.awt.Cursor;

import javax.swing.Icon;
import javax.swing.JToggleButton;
import javax.swing.border.Border;

import de.duehl.math.graph.ged.resources.IconDefinitions;
import de.duehl.math.graph.ged.ui.elements.ModusButtonMouseListener;
import de.duehl.swing.ui.resources.IconLoader;

/**
 * Diese Enum-Klasse steht für den Bearbeitungsmoduszustände der JPanels, in das der Graph
 * eingezeichnet wird.
 *
 * @version 1.01     2019-05-22
 * @author Christian Dühl
 */

public enum EditModus {

    ADD_VERTEX("Ecke einfügen", Cursor.HAND_CURSOR, IconDefinitions.ADD_VERTEX),
    //ADD_UNDIRECTED_EDGE("normale Kante einfügen", Cursor.HAND_CURSOR,
    //        IconDefinitions.ADD_UNDIRECTED_EDGE),
    ADD_EDGE_TO_VERTEX("Kante einfügen: Pfeil hin", Cursor.CROSSHAIR_CURSOR,
            IconDefinitions.ADD_EDGE_TO_VERTEX),
    //ADD_EDGE_FROM_VERTEX("Kante einfügen: Pfeil zurück", Cursor.CROSSHAIR_CURSOR,
    //        IconDefinitions.ADD_EDGE_FROM_VERTEX),
    ADD_BOTH_DIRECTED_EDGE("Kante einfügen: Pfeil hin und zurück", Cursor.CROSSHAIR_CURSOR,
            IconDefinitions.ADD_BOTH_DIRECTED_EDGE),
    MOVE_VERTEX("Ecken verschieben", Cursor.MOVE_CURSOR, IconDefinitions.MOVE_VERTEX),
    INSPECT("Ecken untersuchen", Cursor.DEFAULT_CURSOR, IconDefinitions.INSPECT),
    DELETE_VERTEX("Ecke löschen", Cursor.CROSSHAIR_CURSOR, IconDefinitions.DELETE_VERTEX),
    MERGE_VERTEX("Ecke verschmelzen", Cursor.CROSSHAIR_CURSOR, IconDefinitions.MERGE_VERTEX),
    DELETE_EDGE("Kante löschen", Cursor.CROSSHAIR_CURSOR, IconDefinitions.DELETE_EDGE),
    DELETE_MULTIPLE_EDGES("Alle getroffenen Kanten löschen", Cursor.CROSSHAIR_CURSOR,
            IconDefinitions.DELETE_MULTIPLE_EDGES),
    ;

    /** Beschreibung des Bearbeitungsmodus. */
    private final String description;

    /** Anzuzeigender Maus-Cursor in diesem Bearbeitungsmodus. */
    private final int mouseCursor;

    /** Gibt an, welches Icon zum Bearbeitungsmodus gehört. */
    private final String iconDefinitionPictureIdentifier;

    /** Button für die Oberfläche. */
    private final JToggleButton toggleButton;

    /**
     * Konstruktor.
     *
     * @param description
     *            Beschreibung des Bearbeitungsmodus.
     * @param mouseCursor
     *            Anzuzeigender Maus-Cursor in diesem Bearbeitungsmodus.
     * @param iconDefinitionPictureIdentifier
     *            Gibt an, welches Icon zum Bearbeitungsmodus gehört.
     */
    private EditModus(String description, int mouseCursor, String iconDefinitionPictureIdentifier) {
        this.description = description;
        this.mouseCursor = mouseCursor;
        this.iconDefinitionPictureIdentifier = iconDefinitionPictureIdentifier;

        toggleButton = new JToggleButton(getIcon());
        //toggleButton.setBackground(null);
        //toggleButton.setBorderPainted(false);
        //toggleButton.setContentAreaFilled(false); // kein Unterschied TODO anderswo?
        toggleButton.setBorder(ModusButtonMouseListener.RAISED_BORDER);
    }

    private Icon getIcon() {
        IconLoader loader = new IconDefinitions().createIconLoader();
        return loader.loadIcon(iconDefinitionPictureIdentifier);
    }

    /** Getter für die Beschreibung des Bearbeitungsmodus. */
    public String getDescription() {
        return description;
    }

    /** Getter für den anzuzeigenden Maus-Cursor in diesem Bearbeitungsmodus. */
    public int getMouseCursor() {
        return mouseCursor;
    }

    /** Gibt an, welches Icon zum Bearbeitungsmodus gehört. */
    public String getIconDefinitionPictureIdentifier() {
        return iconDefinitionPictureIdentifier;
    }

    /** Getter für den Button für die Oberfläche. */
    public JToggleButton getToggleButton() {
        return toggleButton;
    }

    /** Setzt den Rahmen für den Button für die Oberfläche. */
    public void setButtonBorder(Border border) {
        toggleButton.setBorder(border);
    }

}
