#------------------------------------------------------------------------------
# Module      : A n i m a t i o n . p m
#------------------------------------------------------------------------------
# Autor       : Christian Dühl
# Created     : 18. Januar 2005
# last change : 19. Januar 2005
# Task        : Animation Class
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# 'Global' private file variables:
#------------------------------------------------------------------------------
my $Debug  = 0;


#------------------------------------------------------------------------------
# Class            : G r a p h : : G E D : : A n i m a t i o n
# derived from     : Graph::GED::Basis
#------------------------------------------------------------------------------
# Author           : Christian Dühl
# Description      : A class modeling the animation of multiple graphs.
# Attributes       : _graphs              - anonym array of Graph2D objects
#                    _histories           - anonym array of Historie objects
#                    filename             - name of the animation file
# Class methods    : new                  - creates a new Animation object
# Object methods   : clone                - clones a Animation object (deep
#                                           copy)
#                    _graphs              - internal Getter / Setter
#                    _histories           - internal Getter / Setter
#                    filename             - Getter / Setter
#                    set_graphs           - set the set of graphs
#                    delete_graphs        - delete all graphs from the set
#                    graph                - setter / getter for one graph
#                                           object via index (index, value)
#                    delete_graph_index   - splices one graph out of the set
#                                           and returns it
#                    insert_graph_index   - splices one graph into the set
#                                           via index (index, object)
#                    push_graph           - Adds a graph to the animation.
#                    pop_graph            - Deletes a graph from the animation
#                    set_histories        - set the set of historys
#                    delete_histories     - delete all historys from the set
#                    history              - setter / getter for one history
#                                           object via index (index, value)
#                    delete_history_index - splices one history out of the set
#                                           and returns it
#                    insert_history_index - splices one history into the set
#                                           (index, object)
#                    push_history         - Adds a history to the animation.
#                    pop_history          - Deletes a history from the
#                                           animation.
#                    clear                - clears everything
#                    number_of_graphs     - returns the number of graphs in the
#                                           Animation.
#                    number_of_histories  - returns the number of histories in
#                                           the Animation.
#                    save                 - saves the animation
#                    load                 - loads the animation
#------------------------------------------------------------------------------
package Graph::GED::Animation;

use strict;
use warnings;

use base qw/Graph::GED::Basis/;


use Graph::GED::Graph2D;
use Graph::GED::Historie;


sub new {
    #--------------------------------------------------------------------------
    # Method       : n e w
    # Class        : Graph::GED::Animation
    #--------------------------------------------------------------------------
    # Author       : Christian Dühl
    # Task         : Creates a new animation object.
    # Parameter    : Class.
    # return value : the new object
    #--------------------------------------------------------------------------
    # 0.0.1 - 2005/01/18 - CD - created
    # 0.0.2 - 2005/01/19 - CD - _histories
    #--------------------------------------------------------------------------

    my $self  = shift;
    my $class = (ref $self) || $self;

    #--------------------------------------------------------------------------
    # Debug output of the parameters:
    #--------------------------------------------------------------------------
    print "Graph::GED::Animation::new\n",
          "\tself       = '$self'\n",
          "\tclass      = '$class'\n",
          "\tParameter  = @_\n",
          if $Debug > 1;


    $self = $class->SUPER::new();


    #--------------------------------------------------------------------------
    # Now we set our own attributes:
    #--------------------------------------------------------------------------
    $self->_graphs   ([]);
    $self->_histories([]);
    $self->filename  ('');

    #--------------------------------------------------------------------------
    # ... and return the new object:
    #--------------------------------------------------------------------------
    return $self;

} # sub Graph::GED::Animation::new


sub clone {
    #--------------------------------------------------------------------------
    # Method       : c l o n e
    # Class        : Graph::GED::Animation
    #--------------------------------------------------------------------------
    # Author       : Christian Dühl
    # Task         : Creates a cloned object (a copy of the actual with deep
    #                copies of arrays and hashes).
    # Parameter    : Perhaps an object, that is handled down by a derived
    #                class, we have to give it to the parent calss.
    #                After that we copy the containers that are handled by
    #                this class.
    # return value : the cloned object
    #--------------------------------------------------------------------------
    # 0.0.1 - 2005/01/18 - CD - created
    # 0.0.2 - 2005/01/19 - CD - _histories
    #--------------------------------------------------------------------------

    my $self = shift;
    my $hndn = shift;
    my $clone;

    print "Graph::GED::Animation::Clone\n" if $Debug;

    #--------------------------------------------------------------------------
    # Do we have to handle an object from a derived class?
    #--------------------------------------------------------------------------
    if (defined $hndn) {
        $clone = $hndn;
    }
    #--------------------------------------------------------------------------
    # If not we create a new object and do a simple copy of its values:
    #--------------------------------------------------------------------------
    else {
        $clone = $self->new();
        %$clone = %$self;
    }

    #--------------------------------------------------------------------------
    # Clone derived parts in the SUPER class:
    #--------------------------------------------------------------------------
    $self->SUPER::clone($clone);

    #--------------------------------------------------------------------------
    # Make a deep copy of own containers:
    #--------------------------------------------------------------------------
    # (Here we have to clone all Elements of the container, because of the
    # (probably) containers in that elements!)
    #--------------------------------------------------------------------------
    my @graphs;
    for (@{$self->_graphs()}) {
        my $graph = $_->clone();
        print "original graph: ", $_    , "\n",
              "cloned graph  : ", $graph, "\n"
              if $Debug > 1;
        push @graphs, $graph;
    }
    $clone->_graphs([ @graphs ]);

    my @histories;
    for (@{$self->_histories()}) {
        my $history = $_->clone();
        print "original history ", $_    , "\n",
              "cloned history  : ", $history, "\n"
              if $Debug > 1;
        push @histories, $history;
    }
    $clone->_histories([ @histories ]);


    return $clone;
} # sub Graph::GED::Animation::clone


#------------------------------------------------------------------------------
# The getter/setter methods:
#------------------------------------------------------------------------------
sub _graphs    { return shift->_property('_graphs',    @_) }
sub _histories { return shift->_property('_histories', @_) }
sub filename   { return shift->_property('filename',   @_) }


#==============================================================================
#
#
#   H E L P E R M E T H O D S
#
#   (only this uses _graphs and _histories)
#
#==============================================================================


sub set_graphs { # set the set of graphs
    $_[0]->_graphs(undef);
    $_[0]->_graphs($_[1]);
} # sub Graph::GED::Animation::set_graphs

sub delete_graphs { # delete all graphs from the set
    $_[0]->_graphs(undef);
    $_[0]->_graphs([]);
} # sub Graph::GED::Animation::delete_graphs

sub graph { # setter / getter for one graph object via index (index, value)
    if (@_ == 3) {
        my $aw = $_[0]->_graphs()->[$_[1]];
        $_[0]->_graphs()->[$_[1]] = $_[2];
        return $aw;
    }
    else {
        return $_[0]->_graphs()->[$_[1]];
    }
} # sub Graph::GED::Animation::graph

sub delete_graph_index { # splices one graph out of the set and returns it
    return splice @{$_[0]->_graphs()}, $_[1], 1;
} # sub Graph::GED::Animation::delete_graph_index

sub insert_graph_index { # splices one graph into the set (index, object)
    splice @{$_[0]->_graphs()}, $_[1], 0, $_[2];
} # sub Graph::GED::Animation::delete_graph_index

sub push_graph {
    push @{$_[0]->_graphs()}, $_[1];
} # sub Graph::GED::Animation::push_graph

sub pop_graph {
    return pop @{ $_[0]->_graphs() };
} # sub Graph::GED::Animation::pop_graph



sub set_histories { # set the set of historys
    $_[0]->_histories(undef);
    $_[0]->_histories($_[1]);
} # sub Graph::GED::Animation::set_histories

sub delete_histories { # delete all historys from the set
    $_[0]->_histories(undef);
    $_[0]->_histories([]);
} # sub Graph::GED::Animation::delete_histories

sub history { # setter / getter for one history object via index (index, value)
    if (@_ == 3) {
        my $aw = $_[0]->_histories()->[$_[1]];
        $_[0]->_histories()->[$_[1]] = $_[2];
        return $aw;
    }
    else {
        return $_[0]->_histories()->[$_[1]];
    }
} # sub Graph::GED::Animation::history

sub delete_history_index { # splices one history out of the set and returns it
    return splice @{$_[0]->_histories()}, $_[1], 1;
} # sub Graph::GED::Animation::delete_history_index

sub insert_history_index { # splices one history into the set (index, object)
    splice @{$_[0]->_histories()}, $_[1], 0, $_[2];
} # sub Graph::GED::Animation::delete_history_index

sub push_history {
    push @{$_[0]->_histories()}, $_[1];
} # sub Graph::GED::Animation::push_history

sub pop_history{
    return pop @{ $_[0]->_histories() };
} # sub Graph::GED::Animation::pop_history


#==============================================================================
#
#
#   M E T H O D S
#
#
#==============================================================================


sub clear {
    #--------------------------------------------------------------------------
    # Method       : c l e a r
    # Class        : Graph::GED::Animation
    #--------------------------------------------------------------------------
    # Author       : Christian Dühl
    # Task         : Deletes all graphs from the animation.
    # Parameter    : none
    # return value : none
    #--------------------------------------------------------------------------
    # 0.0.1 - 2005/01/18 - CD - created
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Animation::clear called with parameters!"
        if scalar @_ > 0;

    print "Graph::GED::Animation::clear\n" if $Debug;

    #--------------------------------------------------------------------------
    # delete arrays:
    #--------------------------------------------------------------------------
    $self->delete_graphs();
    $self->delete_histories();

    #--------------------------------------------------------------------------
    # call basis functionality
    #--------------------------------------------------------------------------
    $self->SUPER::clear();

    #--------------------------------------------------------------------------
    # set standard values:
    #--------------------------------------------------------------------------
    $self->filename('');

} # sub Graph::GED::Animation::clear


sub number_of_graphs {
    #--------------------------------------------------------------------------
    # Method       : number_of_graphs
    # Class        : Graph::GED::Animation
    #--------------------------------------------------------------------------
    # Author       : Christian Dühl
    # Task         : returns the number of graphs in the animation.
    # Parameter    : none
    # return value : none
    #--------------------------------------------------------------------------
    # 0.0.1 - 2005/01/19 - CD - created
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Animation::number_of_graphs called with parameters!"
        if scalar @_ > 0;

    print "Graph::GED::Animation::number_of_graphs\n" if $Debug;

    return scalar @{ $self->_graphs() };

} # sub Graph::GED::Animation::number_of_graphs


sub number_of_histories {
    #--------------------------------------------------------------------------
    # Method       : number_of_histories
    # Class        : Graph::GED::Animation
    #--------------------------------------------------------------------------
    # Author       : Christian Dühl
    # Task         : returns the number of graphs in the animation.
    # Parameter    : none
    # return value : none
    #--------------------------------------------------------------------------
    # 0.0.1 - 2005/01/19 - CD - created
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Animation::number_of_histories called with parameters!" if scalar @_ > 0;

    print "Graph::GED::Animation::number_of_histories\n" if $Debug;

    return scalar @{ $self->_histories() };

} # sub Graph::GED::Animation::number_of_histories


sub save {
    #--------------------------------------------------------------------------
    # Method       : save
    # Class        : Graph::GED::Animation
    #--------------------------------------------------------------------------
    # Author       : Christian Dühl
    # Task         : saves the animation
    # Parameter    : filename
    # return value : none
    #--------------------------------------------------------------------------
    # 0.0.1 - 2005/01/19 - CD - created
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Animation::save called with wrong number of parameters!"
        if scalar @_ != 1;

    my $fn = shift;  # filename

    print "Graph::GED::Animation::save\n" if $Debug;

    open (OUT, ">$fn") or
        die "Kann '$fn' nicht zum Schreiben oeffnen: $!";

    print OUT "<animation>\n";
    for my $g (@{ $self->_graphs() }) {
        print OUT "<graph>\n";
        $g->save_ged3_to_handle(*OUT);
        print OUT "</graph>\n";
    }
    print OUT "</animation>\n";

    close OUT or die "Kann '$fn' nicht schließen: $!";

} # sub Graph::GED::Animation::save


sub load {
    #--------------------------------------------------------------------------
    # Method       : load
    # Class        : Graph::GED::Animation
    #--------------------------------------------------------------------------
    # Author       : Christian Dühl
    # Task         : loads the animation
    # Parameter    : filename
    # return value : none
    #--------------------------------------------------------------------------
    # 0.0.1 - 2005/01/19 - CD - created
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Animation::load called with wrong number of parameters!"
        if scalar @_ != 1;

    my $fn = shift;  # filename


    print "Graph::GED::Animation::load\n" if $Debug;


    $self->clear();
    $self->filename($fn);

    #--------------------------------------------------------------------------
    # Datei einlesen:
    #--------------------------------------------------------------------------
    open (F, $fn) or die "Kann '$fn' nicht oeffnen: $!";
    my $txt;
    {local $/; $txt = <F>;}
    close F or warn $!;

    #--------------------------------------------------------------------------
    # Inhalt des animation-Tags ermitteln:
    #--------------------------------------------------------------------------
    my ($ani) = $txt =~ m~<animation>((?:(?!</animation>).)+)</animation>~s;

    #--------------------------------------------------------------------------
    # Inhalt der graph-Tags ermitteln:
    #--------------------------------------------------------------------------
    while ($ani =~ s~<graph>((?:(?!</graph>).)+)</graph>~~s) {
        my $graph = $1;
        my $g = new Graph::GED::Graph2D;
        $g->load_ged3_from_string($graph);
        $self->push_graph($g);
        $self->push_history(new Graph::GED::Historie);
    }

    return 1;
} # sub Graph::GED::Animation::load


1;


__END__


=head1 NAME

Graph::GED::Animation.pm - A class modeling the animation of multiple graphs.

=head1 METHODS

=head2 class methods

=over

=item C<new>

    Creates an object of this class.

=back

=head2 object methods

=over

=item C<clone>

    Creates a cloned object of this graph.

=item C<_graph>

    Getter / Setter for the set of graphs.

=item C<_history>

    Getter / Setter for the set of histories.

=item C<filename>

    Getter / Setter for the filename.

=item C<set_graphs>

    set the set of graphs

=item C<delete_graphs>

    delete all graphs from the set

=item C<graph>

    setter / getter for one graph object via index (index, value)

=item C<delete_graph_index>

    splices one graph out of the set and returns it

=item C<insert_graph_index>

    splices one graph into the set (index, object)

=item C<push_graph>

    Adds a graph to the animation.

=item C<pop_graph>

    Deletes a graph from the animation

=item C<set_histories>

    set the set of historys

=item C<delete_histories>

    delete all historys from the set

=item C<history>

    setter / getter for one history object via index (index, value)

=item C<delete_history_index>

    splices one history out of the set and returns it

=item C<insert_history_index>

    splices one history into the set (index, object)

=item C<push_history>

    Adds a history to the animation.

=item C<pop_history>

    Deletes a history from the animation.

=item C<clear>

    Deletes all graphs from the animation.

=item C<number_of_graphs>

    Returns the number of graphs in the animation.

=item C<number_of_histories>

    Returns the number of histories in the animation.

=item C<save>

    saves the animation

=item C<load>

    loads the animation

=back


=head1 COPYRIGHT

Copyright 2005 Christian Duehl. All rights reserved.

This library is free software. You can redistribute it and/or
modify it under the same terms as perl itself.

=cut

