#------------------------------------------------------------------------------
# Module      : C o r e . p m
#------------------------------------------------------------------------------
# Autor       : Christian Dühl
# Created     : 19. Januar 2005
# last change : 22. Januar 2005
# Task        : Core data of the GED.
#------------------------------------------------------------------------------

package Graph::GED::Core;

use strict;
use warnings;

use Cwd qw/cwd/; # Ermittlung des aktuellen Arbeitsverzeichnisses

use Graph::GED::Animation;
use Graph::GED::FHist;
use Graph::GED::Graph2D;
use Graph::GED::Historie;

#------------------------------------------------------------------------------
# Derive from Exporter:
#------------------------------------------------------------------------------
use base qw/Exporter/;
our @EXPORT_OK = qw/$Animation
                    $Graph
                    @GitterX
                    @GitterY
                    $History
                    %Linientypen
                    %Konfig
                   /;

require Graph::GED::Gui; # Nicht use, weil sonst Gui die obigen Symbole noch
                         # nicht bekannt sind!

#------------------------------------------------------------------------------
# Configuration:
#------------------------------------------------------------------------------
our %Konfig = (
    debug         => 0,                    # Wert für Debugausgaben,
                                           # je höher der Wert,
                                           # desto mehr wird ausge-
                                           # geben, bei 0 wird
                                           # nichts ausgegeben.
    version       => '0.1.0_beta01',       # Programmversion
    version_datum => '22.01.2005',         # Programmdatum

    mw            => undef,                # Hauptfenster-Widget
    canvas        => undef,                # Canvas-Widget
    menu          => undef,                # Menu-Widget
    menufont      => '{Arial} 8 {normal}', # Schriftart des Menu
    rueck_btn     => undef,                # Rückgängig-Button
    wieder_btn    => undef,                # Wiederholen-Button
    animationskala=> undef,                # Skala für Animation (widget)
    Contextmenu   => [],                   # Array with context menus

    zeige_e_nr    => 0,                    # Eckennummern anzeigen
    zeige_k_nr    => 0,                    # Kantennummern anzeigen

    pfad          => cwd(),                # Pfad zum Laden von
                                           # Dateien.
                                           # (Klappt nicht unter Win
                                           # 98, aber unter Win 2k?!)

    statuszeile   => '',                   # Anzuzeigender Text in
                                           # der Statuszeile

    linientyp     => 'e_ins',              # Mit den Checkbuttons
                                           # verbunden

    startecke     => -1,                   # Startecke der Kante, die
                                           # gerade gezeichnet wird.
    zx            => 0,                    # x-Koordinate und
    zy            => 0,                    # y-Koordinate des
                                           # Zeichnenanfangs

    changed       => ' ',                  # Wurde der Graph seit dem
                                           # Speichern geändert?
                                           # ' ' : nicht geändert
                                           # '*' : geändert

);


#------------------------------------------------------------------------------
# The graph:
#------------------------------------------------------------------------------
our $Graph = new Graph::GED::Graph2D;

#------------------------------------------------------------------------------
# The history:
#------------------------------------------------------------------------------
our $History = new Graph::GED::Historie;

#------------------------------------------------------------------------------
# The animation
#------------------------------------------------------------------------------
our $Animation = new Graph::GED::Animation;
$Animation->push_graph  ($Graph  );
$Animation->push_history($History);

#------------------------------------------------------------------------------
# Line types:
#------------------------------------------------------------------------------
our %Linientypen = (
    e_ins => 'Ecke einfügen',
    e_del => 'Ecke löschen',
    e_mer => 'Ecke mergen',
    e_mv1 => 'Ecke verschieben (schnelle Methode)',
    e_mv2 => 'Ecke verschieben (schöne Methode)',
    k_ins => 'normale Kante einfügen',
    k_del => 'Kante löschen',
    k_dla => 'Kanten löschen',
    k_inh => 'Kante einfügen: Pfeil hin',
    k_inz => 'Kante einfügen: Pfeil zurück',
    k_inb => 'Kante einfügen: Pfeil hin und zurück',
);


#------------------------------------------------------------------------------
# Grid data:
#------------------------------------------------------------------------------
our @GitterX = (); # x-Koordinaten von senkrechten Gitterlinien
our @GitterY = (); # y-Koordinaten von waagerechten Gitterlinien


sub start () {
    #--------------------------------------------------------------------------
    # sub          : s t a r t
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Startet den GED.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1  - 19.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    $Graph->r2d();
    Graph::GED::FHist::load();
    Graph::GED::Gui::start();
} # sub Graph::GED::Core::start

1;


__END__

=head1 NAME

Graph::GED::Core.pm - a package with the core data of the GED

=head1 Variables

=over

=item C<%Konfig>

    Todo list.

=item C<$Graph>

    The graph that is shown in the editor.

=item C<$Animation>

    The animation that is shown in the editor.

=item C<%Linientypen>

    A hash with line types.

=item C<@GitterX>

    An array with the x-coordinates of vertical grid lines.

=item C<@GitterY>

    An array with the y-coordinates of horizontal grid lines.

=back

=head1 FUNCTIONS

=over

=item C<start>

    Starts GED.

=back

=head1 COPYRIGHT

Copyright 2005 Christian Duehl. All rights reserved.

This library is free software. You can redistribute it and/or
modify it under the same terms as perl itself.

=cut
