package de.duehl.math.data;

/*
 * Copyright 2017 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import de.duehl.basics.debug.Assure;

/**
 * Diese Klasse steht für einen Bereich mit minimalem und maximalem Wert.
 *
 * @version 1.01     2017-07-18
 * @author Christian Dühl
 */

public class Range {

    /** Zulässiger Minimalwert des Bereiches. */
    private final int min;

    /** Zulässiger Maximalwert des Bereiches. */
    private final int max;

    /**
     * Konstruktor.
     *
     * @param min
     *            Zulässiger Minimalwert des Bereiches.
     * @param max
     *            Zulässiger Maximalwert des Bereiches.
     */
    public Range(int min, int max) {
        Assure.isTrue(min <= max);
        this.min = min;
        this.max = max;
    }

    /** Getter für den zulässigen Minimalwert des Bereiches. */
    public int getMin() {
        return min;
    }

    /** Getter für den zulässigen Maximalwert des Bereiches. */
    public int getMax() {
        return max;
    }

    /** Überprüft, ob der angegebene Wert im Bereich liegt. */
    public boolean isInRange(int number) {
        return number >= min && number <= max;
    }

    @Override
    public String toString() {
        return "Range [min=" + min + ", max=" + max + "]";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + max;
        result = prime * result + min;
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        Range other = (Range) obj;
        if (max != other.max) {
            return false;
        }
        if (min != other.min) {
            return false;
        }
        return true;
    }

}
