package de.duehl.math.geometry.discrete;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import de.duehl.algorithm.backtracking.AlgorithmSolutionStep;

/**
 * Diese Klasse stellt einen Schritt im Algorithmus zur Erzeugung einer endlichen projektiven Ebene
 * dar.
 *
 * Leider im Moment mit der Notwendigkeit zum Casten. Eigentlich müsste das mit
 * Generics zu vermeiden sein, doch der Versuch dies zu lösen schlug fehl.
 *
 * Der Schritt besteht hier darin, eine Gerade durch einen Punkt laufen zu lassen.
 *
 * @version 1.01     2018-01-02
 * @author Christian Dühl
 */

public class FiniteProjectivePlaneCreatingStep implements AlgorithmSolutionStep {

    private final FiniteProjectiveLine line;

    private final FiniteProjectivePoint point;

    public FiniteProjectivePlaneCreatingStep(FiniteProjectiveLine line, FiniteProjectivePoint point) {
        this.line = line;
        this.point = point;
    }

    public FiniteProjectiveLine getLine() {
        return line;
    }

    public FiniteProjectivePoint getPoint() {
        return point;
    }

    @Override
    public String toString() {
        return "FiniteProjectivePlaneCreatingStep [line=" + line + ", point=" + point + "]";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((line == null) ? 0 : line.hashCode());
        result = prime * result + ((point == null) ? 0 : point.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        FiniteProjectivePlaneCreatingStep other = (FiniteProjectivePlaneCreatingStep) obj;
        if (line == null) {
            if (other.line != null) {
                return false;
            }
        }
        else if (!line.equals(other.line)) {
            return false;
        }
        if (point == null) {
            if (other.point != null) {
                return false;
            }
        }
        else if (!point.equals(other.point)) {
            return false;
        }
        return true;
    }

}
