package de.duehl.math.geometry.discrete;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.util.ArrayList;
import java.util.List;

import de.duehl.basics.text.Text;

/**
 * Diese Klasse stellt ein Symbol dar, welches eine Gerade einer endlichen projektiven Ebene
 * definiert.
 *
 * Angelehnt an das Spiel Dobble wird jede Gerade durch ein Symbol repräsentiert. Jeder
 * Punkt entspricht dann einer Spielkarte mit den acht Symbolen (Ordnung 7 + 1).
 *
 * @version 1.01     2018-01-02
 * @author Christian Dühl
 */

public class Symbol {

    private final String symbol;

    public Symbol(String symbol) {
        this.symbol = symbol;
    }

    boolean hasSymbol(String perhapsSymbol) {
        return symbol.equals(perhapsSymbol);
    }

    @Override
    public String toString() {
        return symbol;
    }

    public static List<Symbol> createSymbols(int n) {
        List<Symbol> symbols = new ArrayList<>();

        String symbolAplhabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZÄÖÜabcdefghijklmnopqrstuvwxyzäöüß";

        if (n <= symbolAplhabet.length()) {
            List<String> bigChars = Text.eachCharFromString(symbolAplhabet);
            for (int index = 0; index < n; ++index) {
                String character = bigChars.get(index);
                Symbol symbol = new Symbol(character);
                symbols.add(symbol);
            }
        }
        else {
            int sybmolAsNumber = 0;
            for (int index = 0; index < n; ++index) {
                String character = Integer.toString(++sybmolAsNumber);
                Symbol symbol = new Symbol(character);
                symbols.add(symbol);
            }

        }

        return symbols;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((symbol == null) ? 0 : symbol.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        Symbol other = (Symbol) obj;
        if (symbol == null) {
            if (other.symbol != null) {
                return false;
            }
        }
        else if (!symbol.equals(other.symbol)) {
            return false;
        }
        return true;
    }

}
