package de.duehl.math.sets;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

import de.duehl.basics.collections.CollectionsHelper;

public class CombinationsTest {

    @Test
    public void empty() {
        List<List<String>> possibleElementsAtEachPositionList = new ArrayList<>();
        Combinations<String> combinations = new Combinations<>(possibleElementsAtEachPositionList);
        List<List<String>> combins = combinations.createCombinations();

        assertEquals(0, combins.size());
    }

    @Test (expected = IllegalArgumentException.class)
    public void doNotUseEmptySubsets() {
        List<List<String>> possibleElementsAtEachPositionList = new ArrayList<>();
        possibleElementsAtEachPositionList.add(
                CollectionsHelper.buildListFrom());
        new Combinations<>(possibleElementsAtEachPositionList);
    }

    @Test
    public void single() {
        List<List<String>> possibleElementsAtEachPositionList = new ArrayList<>();
        possibleElementsAtEachPositionList.add(
                CollectionsHelper.buildListFrom("A"));
        Combinations<String> combinations = new Combinations<>(possibleElementsAtEachPositionList);
        List<List<String>> combins = combinations.createCombinations();

        //System.out.println(combins);
        assertEquals(1, combins.size());
        assertEquals(1, combins.get(0).size());
        assertEquals("A", combins.get(0).get(0));
    }

    @Test
    public void singleValues() {
        List<List<String>> possibleElementsAtEachPositionList = new ArrayList<>();
        possibleElementsAtEachPositionList.add(
                CollectionsHelper.buildListFrom("A"));
        possibleElementsAtEachPositionList.add(
                CollectionsHelper.buildListFrom("B"));
        possibleElementsAtEachPositionList.add(
                CollectionsHelper.buildListFrom("C"));
        Combinations<String> combinations = new Combinations<>(possibleElementsAtEachPositionList);
        List<List<String>> combins = combinations.createCombinations();

        //System.out.println(combins);
        assertEquals(1, combins.size());
        assertEquals(3, combins.get(0).size());
        assertEquals("A", combins.get(0).get(0));
        assertEquals("B", combins.get(0).get(1));
        assertEquals("C", combins.get(0).get(2));
    }

    @Test
    public void doubleMiddleValues() {
        List<List<String>> possibleElementsAtEachPositionList = new ArrayList<>();
        possibleElementsAtEachPositionList.add(
                CollectionsHelper.buildListFrom("A"));
        possibleElementsAtEachPositionList.add(
                CollectionsHelper.buildListFrom("B1", "B2"));
        possibleElementsAtEachPositionList.add(
                CollectionsHelper.buildListFrom("C"));
        Combinations<String> combinations = new Combinations<>(possibleElementsAtEachPositionList);
        List<List<String>> combins = combinations.createCombinations();

        //System.out.println(combins);
        assertEquals(2, combins.size());

        assertEquals(3, combins.get(0).size());
        assertEquals("A", combins.get(0).get(0));
        assertEquals("B1", combins.get(0).get(1));
        assertEquals("C", combins.get(0).get(2));

        assertEquals(3, combins.get(1).size());
        assertEquals("A", combins.get(1).get(0));
        assertEquals("B2", combins.get(1).get(1));
        assertEquals("C", combins.get(1).get(2));
    }

    @Test
    public void threeDoubleValues() {
        List<List<String>> possibleElementsAtEachPositionList = new ArrayList<>();
        possibleElementsAtEachPositionList.add(
                CollectionsHelper.buildListFrom("A1", "A2"));
        possibleElementsAtEachPositionList.add(
                CollectionsHelper.buildListFrom("B1", "B2"));
        possibleElementsAtEachPositionList.add(
                CollectionsHelper.buildListFrom("C1", "C2"));
        Combinations<String> combinations = new Combinations<>(possibleElementsAtEachPositionList);
        List<List<String>> combins = combinations.createCombinations();

        //System.out.println(combins);
        // [ [A1, B1, C1], [A2, B1, C1], [A1, B2, C1], [A2, B2, C1], [A1, B1, C2], [A2, B1, C2],
        //   [A1, B2, C2], [A2, B2, C2]
        //   ]
        assertEquals(8, combins.size());

        assertEquals(3, combins.get(0).size());
        assertEquals("A1", combins.get(0).get(0));
        assertEquals("B1", combins.get(0).get(1));
        assertEquals("C1", combins.get(0).get(2));

        assertEquals(3, combins.get(1).size());
        assertEquals("A2", combins.get(1).get(0));
        assertEquals("B1", combins.get(1).get(1));
        assertEquals("C1", combins.get(1).get(2));

        assertEquals(3, combins.get(2).size());
        assertEquals("A1", combins.get(2).get(0));
        assertEquals("B2", combins.get(2).get(1));
        assertEquals("C1", combins.get(2).get(2));

        assertEquals(3, combins.get(3).size());
        assertEquals("A2", combins.get(3).get(0));
        assertEquals("B2", combins.get(3).get(1));
        assertEquals("C1", combins.get(3).get(2));

        assertEquals(3, combins.get(4).size());
        assertEquals("A1", combins.get(4).get(0));
        assertEquals("B1", combins.get(4).get(1));
        assertEquals("C2", combins.get(4).get(2));

        assertEquals(3, combins.get(5).size());
        assertEquals("A2", combins.get(5).get(0));
        assertEquals("B1", combins.get(5).get(1));
        assertEquals("C2", combins.get(5).get(2));

        assertEquals(3, combins.get(6).size());
        assertEquals("A1", combins.get(6).get(0));
        assertEquals("B2", combins.get(6).get(1));
        assertEquals("C2", combins.get(6).get(2));

        assertEquals(3, combins.get(7).size());
        assertEquals("A2", combins.get(7).get(0));
        assertEquals("B2", combins.get(7).get(1));
        assertEquals("C2", combins.get(7).get(2));
    }

    @Test
    public void oneTwoThreeValues() {
        List<List<String>> possibleElementsAtEachPositionList = new ArrayList<>();
        possibleElementsAtEachPositionList.add(
                CollectionsHelper.buildListFrom("A1"));
        possibleElementsAtEachPositionList.add(
                CollectionsHelper.buildListFrom("B1", "B2"));
        possibleElementsAtEachPositionList.add(
                CollectionsHelper.buildListFrom("C1", "C2", "C3"));
        Combinations<String> combinations = new Combinations<>(possibleElementsAtEachPositionList);
        List<List<String>> combins = combinations.createCombinations();

        //System.out.println(combins);
        // [[A1, B1, C1], [A1, B2, C1], [A1, B1, C2], [A1, B2, C2], [A1, B1, C3], [A1, B2, C3]]
        assertEquals(6, combins.size());

        assertEquals(3, combins.get(0).size());
        assertEquals("A1", combins.get(0).get(0));
        assertEquals("B1", combins.get(0).get(1));
        assertEquals("C1", combins.get(0).get(2));

        assertEquals(3, combins.get(1).size());
        assertEquals("A1", combins.get(1).get(0));
        assertEquals("B2", combins.get(1).get(1));
        assertEquals("C1", combins.get(1).get(2));

        assertEquals(3, combins.get(2).size());
        assertEquals("A1", combins.get(2).get(0));
        assertEquals("B1", combins.get(2).get(1));
        assertEquals("C2", combins.get(2).get(2));

        assertEquals(3, combins.get(3).size());
        assertEquals("A1", combins.get(3).get(0));
        assertEquals("B2", combins.get(3).get(1));
        assertEquals("C2", combins.get(3).get(2));

        assertEquals(3, combins.get(4).size());
        assertEquals("A1", combins.get(4).get(0));
        assertEquals("B1", combins.get(4).get(1));
        assertEquals("C3", combins.get(4).get(2));

        assertEquals(3, combins.get(5).size());
        assertEquals("A1", combins.get(5).get(0));
        assertEquals("B2", combins.get(5).get(1));
        assertEquals("C3", combins.get(5).get(2));
    }

}
