package de.duehl.swing.ui.buttons.painted;

/*
 * Copyright 2017 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.awt.Color;

import de.duehl.swing.ui.buttons.ButtonHelper;

/**
 * Diese Klasse stellt einen kleinen Schalter zur Farbauswahl dar.
 *
 * @version 1.01     2017-02-23
 * @author Christian Dühl
 */

final public class ColoredSquareButton extends PaintedButton {

    private static final long serialVersionUID = -1441507523499749679L;

    /** Farbe des Rechtecks. */
    private Color squareColor;

    /**
     * Konstruktor.
     *
     * @param squareColor
     *            Farbe des Rechtecks.
     */
    public ColoredSquareButton(Color squareColor) {
        super();
        this.squareColor = squareColor;
    }

    /** Getter für die Farbe des Rechtecks. */
    public Color getSquareColor() {
        return squareColor;
    }

    /** Setter für die Farbe des Rechtecks. */
    public void setSquareColor(Color squareColor) {
        this.squareColor = squareColor;
        repaint();
    }

    /**
     * Farbwahl abhängig davon, ob die Maus über dem Button ist.
     *
     * Die gewählte Farbe normalerweise, wenn die Maus darüber kommt, die
     * Komplementärfarbe.
     */
    @Override
    protected void setSymbolColor() {
        graphics2.setColor(squareColor);
        if (getModel().isRollover()) {
            graphics2.setColor(ButtonHelper.antiColor(squareColor));
        }
    }

    /** Zeichnet das Symbol in horizontaler und rechteckiger Anordnung. */
    @Override
    protected void paintHorizontalSymbolRectangle() {
        paintSymbol();
    }

    /** Zeichnet das Symbol in vertikaler und rechteckiger Anordnung. */
    @Override
    protected void paintVerticalSymbolRectangle() {
        paintSymbol();
    }

    /** Zeichnet das Symbol in horizontaler und quadratischer Anordnung. */
    @Override
    protected void paintHorizontalSymbolSquare() {
        paintSymbol();
    }

    /** Zeichnet das Symbol in vertikaler und quadratischer Anordnung. */
    @Override
    protected void paintVerticalSymbolSquare() {
        paintSymbol();
    }

    /**
     * Zeichnet ein farbiges Rechteck. Da es dabei egal ist, ob horizontal oder
     * vertikal, wird es in beiden Fällen verwendet.
     */
    private void paintSymbol() {
        int width = getWidth();
        int height = getHeight();

        if (isForceSquare()) {
            width = Math.min(width, height);
            height = width;
        }

        int delta = 6;

        int firstLeftPixel = (getWidth() - width)/2;
        int left = firstLeftPixel + delta;

        int lastRightPixel = firstLeftPixel + width - 1;
        int right = lastRightPixel - delta;

        int firstUpperPixel = (getHeight() - height)/2;
        int upper = firstUpperPixel + delta;

        int lastLowerPixel = firstUpperPixel + height - 1;
        int lower = lastLowerPixel - delta;

        paintTwoPointRectangle(left, upper, right, lower);
    }

}
