package de.duehl.swing.ui.buttons.painted;

import java.awt.Color;

/*
 * Copyright 2017 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import de.duehl.swing.ui.PixelPoint;
import de.duehl.swing.ui.buttons.ButtonHelper;

/**
 * Diese Klasse stellt einen kleinen Schalter zum Beenden von etwas dar.
 *
 * @version 1.01     2017-12-21
 * @author Christian Dühl
 */

public class XButton extends PaintedButton {

    private static final long serialVersionUID = -1441507523499749679L;

    /** Farbe des Kreuzes. */
    private final Color xColor;

    /** Konstruktor. */
    public XButton() {
        this(Color.BLACK);
    }

    /**
     * Konstruktor.
     *
     * @param xColor
     *            Farbe des Kreuzes.
     */
    public XButton(Color xColor) {
        super();
        this.xColor = xColor;
    }

    /**
     * Farbwahl abhängig davon, ob die Maus über dem Button ist.
     *
     * Die gewählte Farbe normalerweise, wenn die Maus darüber kommt, die
     * Komplementärfarbe.
     */
    @Override
    protected void setSymbolColor() {
        graphics2.setColor(xColor);
        if (getModel().isRollover()) {
            graphics2.setColor(ButtonHelper.antiColor(xColor));
        }
    }

    /** Zeichnet das Symbol in horizontaler und rechteckiger Anordnung. */
    @Override
    protected void paintHorizontalSymbolRectangle() {
        paintSymbol();
    }

    /** Zeichnet das Symbol in vertikaler und rechteckiger Anordnung. */
    @Override
    protected void paintVerticalSymbolRectangle() {
        paintSymbol();
    }

    /** Zeichnet das Symbol in horizontaler und quadratischer Anordnung. */
    @Override
    protected void paintHorizontalSymbolSquare() {
        paintSymbol();
    }

    /** Zeichnet das Symbol in vertikaler und quadratischer Anordnung. */
    @Override
    protected void paintVerticalSymbolSquare() {
        paintSymbol();
    }

    /** Zeichnet das kleine Kreuz zum Schließen. */
    private void paintSymbol() {
        int width = getWidth();
        int height = getHeight();

        if (isForceSquare()) {
            width = Math.min(width, height);
            height = width;
        }

        int delta = 6;

        int firstLeftPixel = (getWidth() - width)/2;
        int left = firstLeftPixel + delta;

        int lastRightPixel = firstLeftPixel + width - 1;
        int right = lastRightPixel - delta;

        int firstUpperPixel = (getHeight() - height)/2;
        int upper = firstUpperPixel + delta;

        int lastLowerPixel = firstUpperPixel + height - 1;
        int lower = lastLowerPixel - delta;

        paintLine(
                new PixelPoint(left,   upper),
                new PixelPoint(right,  lower)
                );
        paintLine(
                new PixelPoint(left,   lower),
                new PixelPoint(right,  upper)
                );
    }

}
