package de.duehl.swing.ui.elements;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JPanel;

import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.buttons.painted.FirstButton;
import de.duehl.swing.ui.buttons.painted.LastButton;
import de.duehl.swing.ui.buttons.painted.NextButton;
import de.duehl.swing.ui.buttons.painted.PreviousButton;

/**
 * Diese Klasse stellt eine Navigation mit vier Schaltern dar, um zum ersten, zum vorigen, zum
 * nächsten und zum letzten Element zu gelangen.
 *
 * @version 1.02     2018-02-13
 * @author Christian Dühl
 */

public class NavigatorPanel extends JPanel {

    private static final long serialVersionUID = 1L;

    private static final Dimension DEFAULT_BUTTON_SIZE = new Dimension(30, 30);

    private final Dimension buttonSize;

    private final JButton firstButton;
    private final JButton previousButton;
    private final JButton nextButton;
    private final JButton lastButton;

    public NavigatorPanel() {
        this(DEFAULT_BUTTON_SIZE);
    }

    public NavigatorPanel(Dimension buttonSize) {
        this.buttonSize = buttonSize;
        setLayout(new GridLayout(1, 0, 3, 3));

        firstButton = new FirstButton();
        previousButton = new PreviousButton();
        nextButton = new NextButton();
        lastButton = new LastButton();

        setButtonDimensions();
        setButtonToolTips();

        add(firstButton);
        add(previousButton);
        add(nextButton);
        add(lastButton);
    }

    private void setButtonDimensions() {
        firstButton.setPreferredSize(buttonSize);
        previousButton.setPreferredSize(buttonSize);
        nextButton.setPreferredSize(buttonSize);
        lastButton.setPreferredSize(buttonSize);
    }

    private void setButtonToolTips() {
        firstButton.setToolTipText("Zeige ersten Datensatz an.");
        previousButton.setToolTipText("Zeige vorigen Datensatz an.");
        nextButton.setToolTipText("Zeige nächsten Datensatz an.");
        lastButton.setToolTipText("Zeige letzten Datensatz an.");
    }

    public void addFirstActionListener(ActionListener actionListener) {
        firstButton.addActionListener(actionListener);
    }

    public void addPreviousActionListener(ActionListener actionListener) {
        previousButton.addActionListener(actionListener);
    }

    public void addNextActionListener(ActionListener actionListener) {
        nextButton.addActionListener(actionListener);
    }

    public void addLastActionListener(ActionListener actionListener) {
        lastButton.addActionListener(actionListener);
    }

    public void setFirstToolTip(String toolTip) {
        firstButton.setToolTipText(toolTip);
    }

    public void setPreviousToolTip(String toolTip) {
        previousButton.setToolTipText(toolTip);
    }

    public void setNextToolTip(String toolTip) {
        nextButton.setToolTipText(toolTip);
    }

    public void setLastToolTip(String toolTip) {
        lastButton.setToolTipText(toolTip);
    }

    public Component getCenteredHorizontal() {
        return GuiTools.centerHorizontal(this);
    }

}
