package de.duehl.swing.ui.key;

/*
 * Copyright 2016 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

/**
 * Oft definiert man in einem Menü Tastaturbefehle, die dann aber nicht
 * funktionieren, wenn der Focus auf z.B. einer JEditorPane liegt.              <br><br>
 *
 * Diese Klasse dient zur Speicherung von einem Tastaturbefehl.
 *
 * @version 1.01     2015-02-17
 * @author Christian Dühl
 */

public class KeyDefinition {

    private final int keyCode;
    private final int modifier;

    public KeyDefinition(final int keyCode, final int modifier) {
        this.keyCode = keyCode;
        this.modifier = modifier;
    }

    public int getKeyCode() {
        return keyCode;
    }

    public int getModifier() {
        return modifier;
    }

    @Override
    public String toString() {
        return "KeyDefinition [keyCode=" + keyCode + ", modifier=" + modifier
                + "]";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + keyCode;
        result = prime * result + modifier;
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        KeyDefinition other = (KeyDefinition) obj;
        if (keyCode != other.keyCode) {
            return false;
        }
        if (modifier != other.modifier) {
            return false;
        }
        return true;
    }

}
